import semver


class BehaviorCloudCacheVersionException(Exception):
    pass


def version_check(manifest_version: str,
                  data_pipeline_version: str,
                  cmin: str,
                  cmax: str):
    mver_parsed = semver.VersionInfo.parse(manifest_version)
    cmin_parsed = semver.VersionInfo.parse(cmin)
    cmax_parsed = semver.VersionInfo.parse(cmax)

    if (mver_parsed < cmin_parsed) | (mver_parsed >= cmax_parsed):
        estr = (f"the manifest has manifest_version {manifest_version} but "
                "this version of AllenSDK is compatible only with manifest "
                f"versions {cmin} <= X < {cmax}. \n"
                "Consider using a version of AllenSDK closer to the version "
                f"used to release the data: {data_pipeline_version}")
        raise BehaviorCloudCacheVersionException(estr)
