from typing import Union
import pathlib
import json
from allensdk.internal.core import DataFile


class EyeTrackingMetadataFile(DataFile):
    """
    Datafile for tracking the metadata file associated with the
    eye tracking camera
    """

    def __init__(self, filepath: Union[str, pathlib.Path]):
        super().__init__(filepath=filepath)

    @staticmethod
    def load_data(filepath: Union[str, pathlib.Path]) -> dict:
        with open(filepath, 'rb') as in_file:
            return json.load(in_file)

    @classmethod
    def file_path_key(cls) -> str:
        return "raw_eye_tracking_video_meta_data"

    @classmethod
    def from_lims(cls):
        raise NotImplementedError(
                "from_lims not yet supported for EyeTrackingMetadataFile")

    @classmethod
    def from_json(
            cls,
            dict_repr: dict) -> "EyeTrackingMetadataFile":
        filepath = dict_repr[cls.file_path_key()]
        return cls(filepath=filepath)
