from pathlib import Path
from typing import Union, Any

from allensdk.internal.api import PostgresQueryMixin

from allensdk.internal.core import DataFile


class EyeTrackingVideo(DataFile):
    """Eye tracking video"""

    @classmethod
    def from_lims(
            cls,
            db: PostgresQueryMixin,
            behavior_session_id: Union[int, str],
            session_type: str = 'OphysSession'
    ) -> "EyeTrackingVideo":
        """

        Parameters
        ----------
        db: PostgresQueryMixin
        behavior_session_id: behavior session id
        session_type: session type the eye tracking video is associated with.
            Either 'OphysSession' or 'EcephysSession'

        Returns
        -------
        `EyeTrackingVideo` instance
        """
        valid_session_types = ('OphysSession', 'EcephysSession')
        if session_type not in valid_session_types:
            raise ValueError(f'Session type must be one of '
                             f'{valid_session_types}')
        query = f"""
                SELECT wkf.storage_directory || wkf.filename AS eye_tracking_file
                FROM behavior_sessions bs
                JOIN ophys_sessions os ON os.id = bs.ophys_session_id
                LEFT JOIN well_known_files wkf ON wkf.attachable_id = os.id
                JOIN well_known_file_types wkft ON wkf.well_known_file_type_id = wkft.id
                WHERE wkf.attachable_type = '{session_type}'
                    AND wkft.name = 'RawEyeTrackingVideo'
                    AND bs.id = {behavior_session_id};
                """  # noqa E501
        filepath = db.fetchone(query, strict=True)
        return cls(filepath=filepath)

    @staticmethod
    def load_data(filepath: Union[str, Path], **kwargs) -> Any:
        return None

    @classmethod
    def from_json(cls, dict_repr: dict) -> "DataFile":
        raise NotImplementedError
