from pynwb import NWBFile

from cachetools import cached, LRUCache
from cachetools.keys import hashkey

from allensdk.core import \
    JsonReadableInterface, LimsReadableInterface, NwbReadableInterface
from allensdk.internal.api import PostgresQueryMixin
from allensdk.core import DataObject


def from_lims_cache_key(cls, db, ophys_experiment_id: int):
    return hashkey(ophys_experiment_id)


class BehaviorSessionId(DataObject, LimsReadableInterface,
                        JsonReadableInterface,
                        NwbReadableInterface,
                        ):
    def __init__(self, behavior_session_id: int):
        super().__init__(name="behavior_session_id", value=behavior_session_id)

    @classmethod
    def from_json(cls, dict_repr: dict) -> "BehaviorSessionId":
        return cls(behavior_session_id=dict_repr["behavior_session_id"])

    @classmethod
    @cached(cache=LRUCache(maxsize=10), key=from_lims_cache_key)
    # TODO should be from_ophys_experiment_id
    def from_lims(
        cls, db: PostgresQueryMixin,
        ophys_experiment_id: int
    ) -> "BehaviorSessionId":
        query = f"""
            SELECT bs.id
            FROM ophys_experiments oe
            -- every ophys_experiment should have an ophys_session
            JOIN ophys_sessions os ON oe.ophys_session_id = os.id
            JOIN behavior_sessions bs ON os.id = bs.ophys_session_id
            WHERE oe.id = {ophys_experiment_id};
        """
        behavior_session_id = db.fetchone(query, strict=True)
        return cls(behavior_session_id=behavior_session_id)

    @classmethod
    def from_ecephys_session_id(
            cls,
            db: PostgresQueryMixin,
            ecephys_session_id: int
    ) -> "BehaviorSessionId":
        query = f"""
            SELECT bs.id
            FROM behavior_sessions bs
            WHERE bs.ecephys_session_id = {ecephys_session_id};
        """
        behavior_session_id = db.fetchone(query, strict=True)
        return cls(behavior_session_id=behavior_session_id)

    @classmethod
    def from_nwb(cls, nwbfile: NWBFile) -> "BehaviorSessionId":
        metadata = nwbfile.lab_meta_data['metadata']
        return cls(behavior_session_id=metadata.behavior_session_id)
