from pynwb import NWBFile

from allensdk.brain_observatory.behavior.data_files import BehaviorStimulusFile
from allensdk.core import DataObject
from allensdk.core import \
    NwbReadableInterface
from allensdk.brain_observatory.behavior.data_files.stimulus_file import \
    StimulusFileReadableInterface


class SessionType(DataObject, StimulusFileReadableInterface,
                  NwbReadableInterface):
    """the stimulus set used"""
    def __init__(self, session_type: str):
        super().__init__(name="session_type", value=session_type)

    @classmethod
    def from_stimulus_file(
            cls,
            stimulus_file: BehaviorStimulusFile) -> "SessionType":
        try:
            stimulus_name = \
                stimulus_file.data["items"]["behavior"]["cl_params"]["stage"]
        except KeyError:
            raise RuntimeError(
                f"Could not obtain stimulus_name/stage information from "
                f"the *.pkl file ({stimulus_file.filepath}) "
                f"for the behavior session to save as NWB! The "
                f"following series of nested keys did not work: "
                f"['items']['behavior']['cl_params']['stage']"
            )
        return cls(session_type=stimulus_name)

    @classmethod
    def from_nwb(cls, nwbfile: NWBFile) -> "SessionType":
        metadata = nwbfile.lab_meta_data['metadata']
        return cls(session_type=metadata.session_type)
