from pynwb import NWBFile

from allensdk.brain_observatory.behavior.data_objects import BehaviorSessionId
from allensdk.brain_observatory.behavior.data_objects.metadata\
    .behavior_metadata.behavior_metadata import \
    BehaviorMetadata
from allensdk.brain_observatory.behavior.data_objects.metadata\
    .behavior_metadata.behavior_session_uuid import \
    BehaviorSessionUUID
from allensdk.brain_observatory.behavior.data_objects.metadata\
    .behavior_metadata.date_of_acquisition import \
    DateOfAcquisition
from allensdk.brain_observatory.behavior.data_objects.metadata\
    .behavior_metadata.equipment import \
    Equipment
from allensdk.brain_observatory.behavior.data_objects.metadata\
    .behavior_metadata.session_type import \
    SessionType
from allensdk.brain_observatory.behavior.data_objects.metadata\
    .behavior_metadata.project_code import \
    ProjectCode
from allensdk.brain_observatory.behavior.data_objects.metadata\
    .behavior_metadata.stimulus_frame_rate import \
    StimulusFrameRate
from allensdk.brain_observatory.behavior.data_objects.metadata\
    .subject_metadata.subject_metadata import \
    SubjectMetadata
from allensdk.core import JsonReadableInterface, NwbReadableInterface


class BehaviorEcephysMetadata(BehaviorMetadata, JsonReadableInterface,
                              NwbReadableInterface):
    def __init__(
            self,
            ecephys_session_id: int,
            date_of_acquisition: DateOfAcquisition,
            subject_metadata: SubjectMetadata,
            behavior_session_id: BehaviorSessionId,
            behavior_session_uuid: BehaviorSessionUUID,
            equipment: Equipment,
            session_type: SessionType,
            stimulus_frame_rate: StimulusFrameRate,
            project_code: ProjectCode = ProjectCode(),
    ):
        super().__init__(
            date_of_acquisition=date_of_acquisition,
            subject_metadata=subject_metadata,
            behavior_session_id=behavior_session_id,
            behavior_session_uuid=behavior_session_uuid,
            equipment=equipment,
            session_type=session_type,
            stimulus_frame_rate=stimulus_frame_rate,
            project_code=project_code,
        )
        self._ecephys_session_id = ecephys_session_id

    @property
    def ecephys_session_id(self) -> int:
        return self._ecephys_session_id

    @classmethod
    def from_json(cls, dict_repr: dict) -> "BehaviorEcephysMetadata":
        behavior_metadata = super().from_json(dict_repr=dict_repr)
        return BehaviorEcephysMetadata(
            ecephys_session_id=dict_repr['ecephys_session_id'],
            date_of_acquisition=DateOfAcquisition(
                behavior_metadata.date_of_acquisition),
            subject_metadata=behavior_metadata.subject_metadata,
            behavior_session_id=behavior_metadata._behavior_session_id,
            behavior_session_uuid=behavior_metadata._behavior_session_uuid,
            equipment=behavior_metadata.equipment,
            session_type=behavior_metadata._session_type,
            project_code=behavior_metadata._project_code,
            stimulus_frame_rate=behavior_metadata._stimulus_frame_rate
        )

    @classmethod
    def from_nwb(cls, nwbfile: NWBFile) -> "BehaviorEcephysMetadata":
        behavior_metadata = super().from_nwb(nwbfile=nwbfile)
        return BehaviorEcephysMetadata(
            ecephys_session_id=int(nwbfile.identifier),
            date_of_acquisition=DateOfAcquisition(
                behavior_metadata.date_of_acquisition),
            behavior_session_id=behavior_metadata._behavior_session_id,
            behavior_session_uuid=behavior_metadata._behavior_session_uuid,
            equipment=behavior_metadata.equipment,
            session_type=behavior_metadata._session_type,
            stimulus_frame_rate=behavior_metadata._stimulus_frame_rate,
            project_code=behavior_metadata._project_code,
            subject_metadata=behavior_metadata.subject_metadata,
        )
