import os
from pathlib import Path

import argschema
import json

from allensdk.brain_observatory.vbn_2022.input_json_writer.schemas import (
    VBN2022InputJsonWriterSchema)

from allensdk.brain_observatory.vbn_2022.input_json_writer.utils import (
    vbn_nwb_config_from_ecephys_session_id_list)


class VBN2022InputJsonWriter(argschema.ArgSchemaParser):
    default_schema = VBN2022InputJsonWriterSchema

    def run(self):

        results = vbn_nwb_config_from_ecephys_session_id_list(
            ecephys_session_id_list=self.args['ecephys_session_id_list'],
            probes_to_skip=self.args['probes_to_skip']
        )

        session_specs = results['sessions']
        msg = results['log']
        self.logger.info("\n\nIrregularities:\n"
                         f"==============\n{msg}")

        for session in session_specs:
            session_id = session['ecephys_session_id']
            json_path = self.args['json_path_lookup'][session_id]
            config = dict()
            config['log_level'] = "INFO"

            str_nwb_path = self.args['nwb_path_lookup'][session_id]
            str_nwb_path = str(str_nwb_path.resolve().absolute())

            os.makedirs(Path(str_nwb_path).parent, exist_ok=True)
            config['output_path'] = str_nwb_path
            config['session_data'] = session
            with open(json_path, 'w') as out_file:
                out_file.write(json.dumps(config, indent=2))
                self.logger.info(f"wrote {json_path.resolve().absolute()}")
